<?php 
// *****************************************************************************
// Copyright 2003-2004 by A J Marston <http://www.tonymarston.net>
// Distributed under the GNU General Public Licence
// *****************************************************************************

class mysql
// this version is for MySQL (using the 'improved' functions)
{

    // member variables
    var $client_info;           // client info (version number)
    var $host_info;             // host info (connection)
    var $server_info;           // server info (version number)
    var $charset;               // character set
    
    var $audit_logging;         // yes/no switch
    var $dbconnect;             // database connection resource
    var $errors;                // array of errors
    var $fieldspec;             // field specifications (see class constructor)
    var $lastpage;              // last available page number in current query
    var $numrows;               // number of rows retrieved
    var $pageno;                // requested page number
    var $rows_per_page;         // page size for multi-row forms
    
    // the following are used to construct an SQL query
    var $sql_select;
    var $sql_count;
    var $sql_from;
    var $sql_where;             // additional selection criteria
    var $sql_groupby;
    var $sql_having;
    var $sql_orderby;
    var $sql_orderby_seq;       // 'asc' or 'desc'
    var $query;                 // completed DML statement
    
    // ****************************************************************************
    // class constructor
    // ****************************************************************************
    function mysql ()
    {
        
    } // mysql
    
    // ****************************************************************************
    function connect ($dbname)
    // establish a connection to the database
    {
        global $dbhost, $dbusername, $dbuserpass, $dbprefix;

        if (!empty($dbprefix)) {
            $dbname = $dbprefix . $dbname;
        } // if
        
        if (!$this->dbconnect) {
            $this->dbconnect   = mysqli_connect($dbhost, $dbusername, $dbuserpass);
            $this->charset     = mysqli_client_encoding($this->dbconnect);
            $this->client_info = mysqli_get_client_info();
            $this->host_info   = mysqli_get_host_info($this->dbconnect);
            $this->server_info = mysqli_get_server_info($this->dbconnect);
        } // if
        if (!$this->dbconnect) {
            return false;
        } elseif (!mysqli_select_db($this->dbconnect, $dbname)) {
            return false;
        } else {
            return true;
        } // if
        
    } // connect
    
    // ****************************************************************************
    function getCount ($dbname, $tablename, $where)
    // get count of records that satisfy selection criteria in $where.
    {
        //DebugBreak();
        $this->errors = array();
        
        if (eregi('^(select )', $where)) {
            // $where starts with 'SELECT' so use it as a complete query
            $this->query = $where;
        } else {
            // does not start with 'SELECT' so it must be a 'where' clause
            $this->query = "SELECT count(*) FROM $tablename WHERE $where";
        } // if
        
        // connect to database
        $this->connect($dbname) or trigger_error($this, E_USER_ERROR);
        
        $result = mysqli_query($this->dbconnect, $this->query) or trigger_error($this, E_USER_ERROR);
        $query_data = mysqli_fetch_row($result);
        
        // if 'GROUP BY' was used then return the number of rows
        if (eregi("group by ", $this->query)) {
            return mysqli_num_rows($result);
        } else {
            return $query_data[0];
        } // if
        
    } // getCount
    
    // ****************************************************************************
    function getData ($dbname, $tablename, $where)
    // get data from a database table using optional 'where' criteria.
    // Results may be affected by $where and $pageno.
    {
        // DebugBreak();
        $this->errors = array();
        
        // connect to database
        $this->connect($dbname) or trigger_error($this, E_USER_ERROR);
        
        $pageno         = $this->pageno;
        $rows_per_page  = $this->rows_per_page;
        $this->numrows  = 0;
        $this->lastpage = 0;
        
        $array = array();
        
        // look for optional SELECT parameters, or default to all fields
        if (empty($this->sql_select)) {
            // the default is all fields
            $select_str = '*';
        } else {
            $select_str = $this->sql_select;
        } // if
        
        // use specified FROM parameters, or default to current table name
        if (empty($this->sql_from)) {
            // the default is current table
            $from_str = $tablename;
        } else {
            $from_str = $this->sql_from;
        } // if
        
        // incorporate optional 'where' criteria
        $where = trim($where);
        if (empty($where)) {
            $where_str = '';
        } else {
            $where_str = "WHERE $where";
        } // if
        
        // incorporate optional GROUP BY parameters
        if (!empty($this->sql_groupby)) {
            $group_str = "GROUP BY $this->sql_groupby";
        } else {
            $group_str = null;
        } // if
        
        // incorporate optional HAVING parameters
        if (!empty($this->sql_having)) {
            $having_str = "HAVING $this->sql_having";
        } else {
            $having_str = null;
        } // if
        
        // count the rows that satisfy this query
        $this->query = "SELECT count(*) FROM $from_str $where_str $group_str $having_str";
        $this->numrows = $this->getCount($dbname, $tablename, $this->query);
        
        if ($this->numrows <= 0) {
            $this->pageno = 0;
            return $array;
        } // if
        
        // incorporate optional sort order
        if (!empty($this->sql_orderby)) {
            $sort_str = "ORDER BY $this->sql_orderby $this->sql_orderby_seq";
        } else {
            $sort_str = '';
        } // if
        
        // calculate the total number of pages from this query
        if ($rows_per_page > 0) {
            $this->lastpage = ceil($this->numrows / $rows_per_page);
        } else {
            $this->lastpage = 1;
        } // if
        
        // ensure pageno is within range
        if ($pageno == '' OR $pageno <= '1') {
            $pageno = 1;
        } elseif ($pageno > $this->lastpage) {
            $pageno = $this->lastpage;
        } // if
        $this->pageno = $pageno; 
        
        // set the limit values to retrieve the specified page number
        if ($rows_per_page > 0) {
            $limit_str = 'LIMIT ' . ($pageno - 1) * $rows_per_page . ',' . $rows_per_page;
        } else {
            $limit_str = '';
        } // if
        
        // DebugBreak();
        // build the query string and run it
        $this->query = "SELECT $select_str FROM $from_str $where_str $group_str $having_str $sort_str $limit_str";
        $result = mysqli_query($this->dbconnect, $this->query) or trigger_error($this, E_USER_ERROR);
        
        // convert result set into a simple associative array for each row
        while ($row = mysqli_fetch_assoc($result)) {
            $array[] = $row;
        } // while
        
        mysqli_free_result($result);
        
        return $array;
        
    } // std_getData
    
    // ****************************************************************************
    function getErrors ()
    {
        return $this->errors;
        
    } // getErrors
    
    // ****************************************************************************
    function getErrorString () 
    // return string containing details of last error.
    {
        $string = 'MySQL error: ' . mysqli_errno($this->dbconnect) 
                                  . ' - ' 
                                  . mysqli_error($this->dbconnect);
        
        return $string;
        
    } // getErrorString
    
    // ****************************************************************************
    function getLastPage () 
    // return the last page number for retrieved rows.
    {
        return (int)$this->lastpage;
        
    } // getLastPage
    
    // ****************************************************************************
    function getNumRows () 
    // return the number of rows retrived for the current page.
    {
        return (int)$this->numrows;
        
    } // getNumRows
    
    // ****************************************************************************
    function getPageNo () 
    // get current page number to be retrieved for a multi-page display
    {
        if (empty($this->pageno)) {
            return 0;
        } else {
            return (int)$this->pageno;
        } // if
        
    } // getPageNo
    
    // ****************************************************************************
    function getPkeyNames () 
    // return the list of primary key fields in this table.
    {
        $array = getPkeyNames($this->fieldspec);
        
        return $array;
        
    } // getPkeyNames
    
    // ****************************************************************************
    function getQuery () 
    // return the last query string that was used
    {
        return $this->query;
        
    } // getQuery
    
    // ****************************************************************************
    function setOrderBy ($sql_orderby) 
    // this allows a sort order to be specified (see getData)
    {
        $this->sql_orderby = trim($sql_orderby);
        
    } // setOrderBy
    
    // ****************************************************************************
    function setOrderBySeq ($sql_orderby_seq) 
    // this allows a sort sequence ('asc' or 'desc') to be set (see getData)
    {
        $this->sql_orderby_seq = trim($sql_orderby_seq);
        
    } // setOrderBySeq
    
    // ****************************************************************************
    function setPageNo ($pageno = '1') 
    // this allows a particular page number to be selected (see getData)
    {
        $this->pageno = (int)$pageno;
        
    } // setPageNo
    
    // ****************************************************************************
    function setRowsPerPage ($rows_per_page) 
    // this allows the default value to be changed
    {
        if ($rows_per_page > 0) {
            $this->rows_per_page = (int)$rows_per_page;
        } // if
        
    } // setRowsPerPage
    
    // ****************************************************************************
    function updateRecord ($dbname, $tablename, $fieldarray, $oldarray)
    // update a record using the contents of $fieldarray.
    {
        // DebugBreak();
        $this->errors = array();
        
        // connect to database
        $this->connect($dbname) or trigger_error($this, E_USER_ERROR);
        
        // get field specifications for this database table
        $fieldspec = $this->fieldspec;
        
        // remove any non-database fields from input array
        foreach ($fieldarray as $field => $fieldvalue) {
            // check that $field exists in $fieldspec array
            if (!array_key_exists($field, $fieldspec)) {
                // it does not (like the SUBMIT button, for example), so remove it
                unset ($fieldarray[$field]);
            } // if
        } // foreach
        
        $pkey_array = $this->getPkeyNames();
        
        // build 'where' string using values for primary key
        $where = array2where($fieldarray, $pkey_array);
                
        // remove any values that have not changed
        $fieldarray = getChanges($fieldarray, $oldarray);
        
        // build update string from non-pkey fields
        $update = null;
        foreach ($fieldarray as $item => $value) {
            // require this item if it IS NOT part of primary key
            if (!in_array($item, $pkey_array)) {
                if (is_null($value)) {
                    // null entries are set to NULL, not '' (there is a difference!)
                    $update .= "$item=NULL,";
                } else {
                    // change to the new value
                    $update .= "$item='$value', ";
                } // if
            } // if
        } // foreach
        
        // strip trailing comma
        if (empty($update)) {
            return $fieldarray;
        } else {
            $update = rtrim($update, ', ');
        } // if
        
        // append WHERE clause to SQL query
        $this->query = "UPDATE $tablename SET $update WHERE $where";
        $result = mysqli_query($this->dbconnect, $this->query) or trigger_error($this, E_USER_ERROR);
        
        // get count of affected rows as there may be more than one
        $this->numrows = mysqli_affected_rows($this->dbconnect);
        
        return;
        
    } // updateRecord
    
    // ****************************************************************************
    function __sleep () 
    // perform object clean-up before serialization
    {
        // get associative array of class variables
        $object_vars = get_object_vars($this);
        
        // remove unwanted variables
        unset($object_vars['data_unformatted']);
        
        // convert to indexed array
        $object_vars = array_keys($object_vars);
        
        return $object_vars;
        
    } // __sleep
    
// ****************************************************************************
} // end class
// ****************************************************************************

?>
