<?php 
// *****************************************************************************
// Copyright 2003-2004 by A J Marston <http://www.tonymarston.net>
// *****************************************************************************

// This file contains generic functions
//DebugBreak();

// set configuration options
;$result = ini_set('register_globals', '0');
;$result = ini_set('magic_quotes_gpc', '0');
$result = ini_set('session.use_cookies', '1');
$result = ini_set('session.use_only_cookies', '0');
$result = ini_set('session.use_trans_sid', '0');
// remove 'form=' as this makes XHTML forms invalid
$result = ini_set('url_rewriter.tags', 'a=href,area=href,frame=src,input=src,fieldset=');
$result = ini_set('arg_separator.output', '&amp;');

// get session name (may not be PHPSESSID)
global $session_name;
$session_name = session_name();

$dbms_engine = array();
$dbhost      = "localhost";
$dbusername  = "tony";
$dbuserpass  = "tony";
$dbprefix    = null;

set_error_handler('errorHandler');

$xslt_error = null;

// ****************************************************************************
function errorHandler ($errno, $errstr, $errfile, $errline, $errcontext)
// If the error condition is E_USER_ERROR or above then abort.
{
    if ($errno != E_STRICT) {
        if (function_exists('debugbreak')) {
            DebugBreak();
        } // if
    } // if
    
    switch ($errno) {
        case E_USER_WARNING:
        case E_USER_NOTICE:
        case E_WARNING:
        case E_NOTICE:
        case E_CORE_WARNING:
        case E_COMPILE_WARNING:
        case E_STRICT;
            return;
        case E_USER_ERROR:
        case E_ERROR:
        case E_PARSE:
        case E_CORE_ERROR:
        case E_COMPILE_ERROR:
        default:
            break;
    } // switch
    
    if (isset($errcontext['this'])) {
        if (is_object($errcontext['this'])) {
            $classname   = get_class($errcontext['this']);
            $parentclass = get_parent_class($errcontext['this']);
        } // if
    } // if
    
    if (!isset($_SESSION)) {
        session_start();
    } // if
    
    //if (eregi('^(sql)$', $errstr)) {
    if (eregi('^(object)$', $errstr)) {
        if (isset($errcontext['this'])) {
            // retrieve error details from DML object
            $query  = $errcontext['this']->getQuery();
            $errstr = $errcontext['this']->getErrorString();
        } else {
            $query  = null;
        } // if
    } else {
        $query = null;
    } // if
    
    $errorstring = "<h2>" . date('Y-m-d H:i:s') . "</h2>\n";
    $errorstring .= "<p>Fatal Error: $errstr (# $errno).</p>\n";
    if ($query) $errorstring .= "<p>SQL query: $query</p>\n";
    $errorstring .= "<p>Error in line $errline of file '$errfile'.</p>\n";
    $errorstring .= "<p>Script: '{$_SERVER['PHP_SELF']}'.</p>\n";
    
    if (isset($classname)) {
        $errorstring .= "<p>Object/Class: '$classname', Parent Class: '$parentclass'.</p>\n";
    } // if
    $errorstring .= "<p>Remote Address: {$_SERVER['REMOTE_ADDR']}</p>\n"; 
    
    // NOTE: this test is only here because my development PC does not have a working mail function
    if ($_SERVER['SERVER_NAME'] <> 'localhost') {
        // remove all html tags
        $search = array('<br>', '<p>', '</p>', '<h2>', '</h2>');
        $mailstring = str_replace($search, '', $errorstring); 
        // Send the error to the administrator by email
        error_log($mailstring, 1, $_SERVER['SERVER_ADMIN']);
    } // if
    
    echo "<h2>This application has encountered an unrecoverable error</h2>\n";
    echo "<p>The following has been reported to the administrator:</p>\n";
    echo "<b><font color='red'>\n$errorstring\n</b></font>";
    
    // remove line breaks
    $errorstring = str_replace("\n", "", $errorstring);
    $errorstring .= "\n"; // except the last one
    
    // add message to log file
    $logfile = $_SERVER['DOCUMENT_ROOT'] . '/errorlog.html';
    error_log($errorstring, 3, $logfile);
    
    // Stop the system
    session_unset();
    session_destroy();
    die();
    
} // errorHandler

// *****************************************************************************
function XML_errorHandler ($errno, $errstr, $errfile, $errline, $errcontext)
// deal with errors from XML or XSL functions.
{
    // pass these details to the standard error handler
    errorHandler (E_USER_ERROR, $errstr, $errfile, $errline, $errcontext);

} // XML_errorHandler

// *****************************************************************************
function array2where ($inputarray, $fieldspec = '')
// turn an array of 'name=value' pairs into an SQL 'where' clause.
// $fieldspec may be in format 'n=name' or 'name=array'.
{
    // DebugBreak();
    if (!is_array($inputarray)) {
        return '';  // input is empty, so output is empty
    } // if
    
    // if $input array is indexed by rownum use only row zero
    if (is_long(key($inputarray))) {
        $array = $inputarray[0];
        $inputarray = $array;
    } // if
    
    // if $fieldspec is empty use $inputarray
    if (empty($fieldspec)) {
        $fieldspec = $inputarray;
    } // if
    
    // if $fieldspec is in format 'n=name' change it to 'name=n'
    if (is_long(key($fieldspec))) {
        $fieldspec = array_flip($fieldspec);
    } // if
    
    $where = '';
    foreach ($inputarray as $fieldname => $fieldvalue) {
        // exclude fields not contained in $fieldspec (such as SUBMIT button)
        if (array_key_exists($fieldname, $fieldspec)) {
            if (eregi('^(=| LIKE | IS )', $fieldvalue, $regs)) {
                // $fieldvalue contains an operator
                if ($regs[0] == '=') {
                    $namevalue = "$fieldname$fieldvalue";
                } else {
                    $namevalue = "$fieldname $fieldvalue";
                } // if
            } else {
                // $fieldvalue does not contain operator, assume '='
                $namevalue = "$fieldname='$fieldvalue'";
            } // if
            if (empty($where)) {
                $where = $namevalue;
            } else {
                $where .= " AND $namevalue";
            } // if
        } // if
    } // foreach
    
    if (!empty($where)) {
        $where = '(' . $where . ')'; // surround with brackets
    } // if
    
    return $where;
    
} // array2where

// ****************************************************************************
function array_update ($array1, $array2)
// update contents of $array1 from contents of $array2.
// Note: this is different from a merge which will add new fields into $array1
// if they did not previously exist, which is not what I want. This version
// will not create any items in $array1 which did not previously exist.
{
    foreach ($array1 as $fieldname => $fieldvalue) {
        if (array_key_exists($fieldname, $array2)) {
            $array1[$fieldname] = $array2[$fieldname];
        } // if
    } // foreach
    
    return $array1;
    
} // array_update

// ****************************************************************************
function getPkeyNames ($fieldspec)
// get array of field names which form the primary key in this database table
{
    $pkey_array = array();
    
    foreach ($fieldspec as $field => $spec) {
        // look for keyword 'pkey' in field specifications
        if (isset($spec['pkey'])) {
            $pkey_array[] = $field;
        } // if
    } // foreach
    
    return $pkey_array;
    
} // getPkeyNames

// ****************************************************************************
function getChanges ($newarray, $oldarray)
// compare two arrays of 'name=value' pairs and remove items from $newarray
// which have the same value in $oldarray.
{
    // step through each 'item=value' entry in $newarray
    foreach ($newarray as $item => $value) {
        // remove if item with same value exists in $oldarray
        if (stripslashes($value) == $oldarray[$item]) {
            unset ($newarray[$item]);
        } // if
    } // foreach
    
    return $newarray;
    
} // getChanges

// ****************************************************************************
function initSession()
// initialise session data
{
    global $button_text;    // short text for menu bar
    global $errors;         // array of error messages
    global $message;        // optional array of messages
    global $selection;      // a selection of rows made in one screen that is passed back to another
    global $script_count;   // number of times this script has been called in this cycle
    global $script_start;   // start time
    global $script_vars;    // variables belonging to current script
    global $session_name;   // session name, to allow multiple sessions from the same PC
    global $title;          // script title
    global $task_id;        // current task identity on menu database
    global $where;          // selection criteria, used in SQL query
    
    //DebugBreak();
    // continue with existing session, or start a new one
    if (!isset($_SESSION)) {
        if (isset($session_name)) {
            session_name($session_name); // set the session name
        } // if
        session_start(); // open/reopen session
    } // if
    
    // record the time at which this script started
    // (end time is determined in function XSLTransform)
    if (!isset($_SESSION['script_start'])) {
        list($usec, $sec) = explode(' ', microtime());
        $_SESSION['script_start'] = (float) $sec + (float) $usec;
    } // if
    
    $errors = array();
    
    // look for session variables associated with this script
    if (isset($_SESSION[$_SERVER['PHP_SELF']])) {
        $script_vars = $_SESSION[$_SERVER['PHP_SELF']];
    } else {
        $script_vars = array();
    } // if
    
    // keep count of how many times this script has run
    if (isset($script_vars['script_count'])) {
        $script_count = $script_vars['script_count'];
    } else {
        $script_count = 0;
    } // if
    $script_count++;
    $script_vars['script_count'] = $script_count;
    
    $task_id = basename($_SERVER['PHP_SELF']);
    
    if (isset($script_vars['title'])) {
        $title = $script_vars['title'];
    } // if
    
    // check for message(s) returned by previous script
    if (isset($script_vars['message'])) {
        if (is_array($script_vars['message'])) {
            $message = $script_vars['message'];
        } else {
            // convert simple string into an array
            $message[] = $script_vars['message'];
        } // if
        unset($script_vars['message']);
    } // if
    
    // check for SQL 'where' string to be used within this script
    // (this is usually passed down by the parent script)
    if (isset($script_vars['where'])) {
        $where = $script_vars['where'];
    } // if
    
    // save values in session data
    $_SESSION[$_SERVER['PHP_SELF']] = $script_vars;
    
    return;
    
} // initSession

// ****************************************************************************
function isPkeyComplete($where, $pkey)
// check that $where contains all fields for the primary key
{
    if (is_string($where)) {
        // convert string into array
        $fieldarray = where2array($where); 
        // use only first row
        $fieldarray = $fieldarray[0];
    } else {
        $fieldarray = $where;
    } // if
    
    foreach ($pkey as $fieldname) {
        if (!array_key_exists($fieldname, $fieldarray)) {
            return 'Primary key is not complete - check selection';
        } // if
    } // foreach
    
    return;
    
} // isPkeycomplete

// ****************************************************************************
function where2array ($where)
// change an SQL 'where' clause into an array of field names and values
// $where is in the format: (name='value' AND name='value' AND ...)
// or possibly: (name='value' AND name='value') OR (name='value' AND name='value') OR ...
{
    // DebugBreak();
    // if input string is empty there is nothing to do
    if (empty($where)) return $where;
    
    // ensure keywords are in upper case
    $where = eregi_replace(' or ', ' OR ', $where);
    $where = eregi_replace(' and ', ' AND ', $where);
    $where = eregi_replace(' like ', ' LIKE ', $where);
    $where = eregi_replace(' is ', ' IS ', $where);
    
    // there may be several clauses separated by 'OR' where
    // each one will be regarded as a separate row
    $OR_array = explode(' OR ', $where);
    
    $array = array();
    $row = 0;
    foreach ($OR_array as $where1) {
        // strip off outer '(' and ')'
        $where1 = trim($where1, '( )'); 
        // separate each 'name=value' pair
        $where2 = explode(' AND ', $where1); 
        // process each set of 'name=value' strings
        $where3 = null;
        foreach ($where2 as $namevalue) {
            // the operator may be '=' or 'LIKE'
            eregi('(=| LIKE | IS )', $namevalue, $operator); 
            // separate the fieldname and fieldvalue from the operator
            // (limit = 2 to stop at the first '=' sign as $fieldvalue may also contain an '='sign)
            list($fieldname, $fieldvalue) = split($operator[0], $namevalue, 2); 
            // add to array for current row
            $array[$row][trim($fieldname)] = "$operator[0]$fieldvalue";
        } // foreach
        $row++;
    } // foreach
    
    return $array;
    
} // where2array

// ****************************************************************************
function addData2XMLdoc ($doc, $root, $parent, $dbobject, &$errors)
// add contents of object's data array to current XML document.
// NOTE: $parent may or may not be the same as $root.
// NOTE: $errors is passed BY REFERENCE as it may be updated.
{
    // DebugBreak();
    if (!is_object($dbobject)) {
        return; // something is wrong, so stop now
    } // if
    
    if (empty($errors)) {
        $errors = array();
    } else {
        // array is not indexed by rownum, so set it to row zero
        if (!is_long(key($errors))) {
            $array[0] = $errors;
            $errors = $array;
        } // if
    } // if
    
    // get field specifications for current object (inluding item access details)
    $fieldspec = $dbobject->getFieldSpec();
    
    // get data array for current object (indexed by rownum)
    $objectdata = $dbobject->getFieldArray();
    
    // process the data one row at a time
    foreach ($objectdata as $rownum => $rowdata) {
        // add a node to identify the name of this database table
        $node = $doc->createElement(strtolower($dbobject->getTableName()));
        $node = $parent->appendChild($node);
        if (!is_array($rowdata)) {
            // this is not an array, so output as data now
            // insert as text element for the current node
            $value = $doc->createTextNode($rowdata);
            $value = $node->appendChild($value);
        } else {
            // add a child node for each field
            foreach ($rowdata as $fieldname => $fieldvalue) {
                $child = $doc->createElement($fieldname);
                $child = $node->appendChild($child);
                if (isset($fieldspec[$fieldname])) {
                    // get field specifications
                    $spec = $fieldspec[$fieldname]; 
                    // add field size as an attribute
                    if (isset($spec['size'])) {
                        $child->setAttribute('size', $spec['size']);
                    } // if 
                    // look for noedit attribute
                    if (isset($spec['noedit'])) {
                        $child->setAttribute('noedit', 'y');
                    } else {
                        // look for primary key attribute
                        if (isset($spec['pkey'])) {
                            $child->setAttribute('pkey', 'y');
                        } // if
                        if (isset($spec['required'])) {
                            $child->setAttribute('required', 'y');
                        } // if 
                        // option for password fields
                        if (isset($spec['password'])) {
                            $child->setAttribute('password', 'y');
                        } // if
                    } // if 
                    // look for nodisplay/hidden attribute
                    if (isset($spec['nodisplay'])) {
                        $child->setAttribute('nodisplay', 'y');
                    } else {
                        // if an error message exists put it into the error attribute
                        if (!empty($errors) and is_array($errors[$rownum])) {
                            if (array_key_exists($fieldname, $errors[$rownum])) {
                                $child->setAttribute("error", $errors[$rownum][$fieldname]);
                                unset($errors[$rownum][$fieldname]);
                            } // if
                        } // if
                    } // if
                    if (isset($spec['type'])) {
                        if ($spec['type'] == 'boolean') {
                            $child->setAttribute('control', 'boolean');
                        } // if
                    } // if
                    if (isset($spec['control'])) {
                        // option for dropdown fields
                        if ($spec['control'] == 'dropdown') {
                            $child->setAttribute('control', 'dropdown');
                            $child->setAttribute('optionlist', $spec['optionlist']);
                        } // if 
                        // option for filename fields
                        if ($spec['control'] == 'filepicker') {
                            $child->setAttribute('control', 'filepicker');
                            if (isset($spec['filepicker'])) {
                                $child->setAttribute('filepicker', "task#{$spec['filepicker']}");
                            } // if
                            if (isset($spec['image'])) {
                                $child->setAttribute('image', 'y');
                                if (isset($spec['imagewidth'])) {
                                    $child->setAttribute('imagewidth', $spec['imagewidth']);
                                } // if
                                if (isset($spec['imageheight'])) {
                                    $child->setAttribute('imageheight', $spec['imageheight']);
                                } // if
                            } // if
                        } // if 
                        // options for multiline fields
                        if ($spec['control'] == 'multiline') {
                            $child->setAttribute('control', 'multiline');
                            $child->setAttribute('cols', $spec['cols']);
                            $child->setAttribute('rows', $spec['rows']);
                        } // if 
                        // option for popup fields
                        if ($spec['control'] == 'popup') {
                            $child->setAttribute('control', 'popup');
                            $child->setAttribute('foreignfield', $spec['foreignfield']);
                            if (isset($spec['popupname'])) {
                                $child->setAttribute('popupname', "task#{$spec['popupname']}");
                            } // if
                        } // if 
                        // option for radio buttons
                        if ($spec['control'] == 'radiogroup') {
                            $child->setAttribute('control', 'radiogroup');
                            $child->setAttribute('optionlist', $spec['optionlist']);
                            $child->setAttribute('align', $spec['align']);
                        } // if
                    } // if
                } else {
                    // field does not exist in $fieldspec, so make it non-editable
                    $child->setAttribute('noedit', 'y');
                } // if
                // insert as text element for the current node
                $value = $doc->createTextNode($fieldvalue);
                $value = $child->appendChild($value);
            } // foreach
        } // if
        if (empty($errors[$rownum])) {
            // no outstanding errors for this row - delete row pointer
            unset($errors[$rownum]);
        } // if
    } // foreach
    
    // return the node created here as it may be used as the
    // parent node for subsequent input
    return $node;
    
} // addData2XMLdoc

// ****************************************************************************
function buildXML ($xml_objects, $errors = null, $message = null)
// build XML document using the database objects identified in $xml_objects
{
    //DebugBreak();
    // test that DOM XML funcions are available
    if (!class_exists('DomDocument')) {
        trigger_error('DOM functions are not available.', E_USER_ERROR);
    } // if
    
    global $act_buttons;
    global $current_menu;
    global $current_task;
    global $menu_buttons;
    global $mode;
    global $nav_buttons;
    global $pagination;
    global $screen;
    global $scrolling;
    global $task_id;
    global $xsl_file;
    
    // create a new XML document
    $xml_doc = new DomDocument('1.0');
    
    // add root node
    $node_array['root'] = $xml_doc->createElement('root');
    $root = $xml_doc->appendChild($node_array['root']);
    
    foreach ($xml_objects as $seq => $object_data) {
        $node = key($object_data);
        $object = $object_data[$node];
        if (!array_key_exists($node, $node_array)) {
            trigger_error("Node $node does not exist in node array", E_USER_ERROR);
        } // if 
        // add data from this object to XML document
        $child = addData2XMLdoc($xml_doc, $node_array['root'], $node_array[$node], $object, $errors); 
        // add this object's name to $node_array
        $node_array[$object->getTableName()] = $child;
    } // foreach
    
    // insert action buttons
    if (isset($act_buttons)) {
        setActBar($xml_doc, $root, $act_buttons);
    } // if
    
    if (isset($message)) {
        // add message to error array for processing
        if (is_array($message)) {
            $errors = array_merge($message, $errors);
        } else {
            $errors[] = $message;
        } // if
    } // if
    
    if (!empty($errors)) {
        if (!is_array($errors)) {
            // convert string into an array
            $errors = (array)$errors;
        } // if
        if (!is_long(key($errors))) {
            // array is not indexed by rownum, so set it to row zero
            $array[0] = $errors;
            $errors = $array;
        } // if
    } // if
    
    // there may be errors for fields which are not in the form
    // so write them to separate error lines
    if (!empty($errors)) {
        // add node to hold error lines
        $occ = $xml_doc->createElement('message');
        $occ = $root->appendChild($occ);
        foreach ($errors as $rownum => $rowdata) {
            if (!is_array($rowdata)) {
                // convert string into an array
                $rowdata = (array)$rowdata;
            } // if
            foreach ($rowdata as $id => $msg) {
                $child = $xml_doc->createElement('line');
                $child = $occ->appendChild($child);
                $value = $xml_doc->createTextNode($msg);
                $value = $child->appendChild($value);
            } // foreach
        } // foreach
    } // if
    
    if (isset($screen)) {
        // process file containing screen structure
        $xsl_file = setScreenStructure($xml_doc, $root, $screen, $xsl_file);
    } // if
    
    // get completed xml document
    //$xml_string = $xml_doc->dump_mem(true, 'ISO-8859-1');
    $xml_string = $xml_doc->saveXML();
    
    if ($_SERVER['SERVER_NAME'] == 'localhost') {
        // write XML data to a disk file in XSL subdirectory for debugging
        $fname = './xsl/' . basename($_SERVER['PHP_SELF']) . '.xml' ;
        $fp = fopen($fname, 'w');
        $result = fwrite($fp, $xml_string);
        fclose($fp);
    } // if
    
    // transform XML document using XSL file
    XSLTransform($xml_doc, $xsl_file);
    
    return;
    
} // buildXML

// ****************************************************************************
function setActBar ($doc, $root, $act_buttons)
// add contents of $act_buttons to the current XML document
{
    // add element containing action buttons
    $occ = $doc->createElement('actbar');
    $occ = $root->appendChild($occ);
    
    foreach ($act_buttons as $button => $label) {
        // add each button to the xml document
        $child = $doc->createElement('button');
        $child = $occ->appendChild($child);
        $child->setAttribute('id', $button);
        $value = $doc->createTextNode($label);
        $value = $child->appendChild($value);
    } // foreach
    
    return;
    
} // setActBar

// ****************************************************************************
function setScreenStructure ($xml_doc, $root, $screen, $xsl_file)
// extract screen structure from $screen and insert details into XML document.
{
    // DebugBreak();
    if (isset($screen['xsl_file'])) {
        $xsl_file = $screen['xsl_file'];
    } // if
    
    // define node to contain all structure elements
    $occ = $xml_doc->createElement('structure');
    $occ = $root->appendChild($occ);
    
    // structure may contain one or more tables, so step through each one
    foreach ($screen['tables'] as $zone => $id) {
        // add this table name as a child to the structure element
        $table = $xml_doc->createElement($zone);
        $table = $occ->appendChild($table);
        $table->setAttribute('id', $id); 
        // set column values for this table, if present
        if (isset($screen[$zone]['columns'])) {
            $columns = $xml_doc->createElement('columns');
            $columns = $table->appendChild($columns);
            foreach ($screen[$zone]['columns'] as $row => $array) {
                foreach ($array as $attrname => $attrvalue) {
                    if ($attrname != strtolower('nosort')) {
                        $column = $xml_doc->createElement('column');
                        $column = $columns->appendChild($column);
                        $column->setAttribute($attrname, $attrvalue);
                    } // if
                } // foreach
            } // foreach
        } // if
        
        // set field names and field labels for this table
        $fields = $xml_doc->createElement('fieldlist');
        $fields = $table->appendChild($fields);
        $i = 0;
        foreach ($screen[$zone]['fields'] as $fieldname => $fieldlabel) {
            $field = $xml_doc->createElement('field');
            $field = $fields->appendChild($field);
            $field->setAttribute('id', $fieldname);
            $field->setAttribute('label', $fieldlabel); 
            // look for optional 'nosort' attribute for this column
            if (isset($screen[$zone]['columns'][$i]['nosort'])) {
                $nosort = $screen[$zone]['columns'][$i]['nosort'];
                if (!empty($nosort)) {
                    $field->setAttribute('nosort', $nosort);
                } // if
            } // if
            $i++;
        } // foreach
    } // foreach
    
    return $xsl_file;
    
} // setScreenStructure

// ****************************************************************************
function XSLTransform ($xml_doc, $xsl_file)
// transform XML into HTML using XSL file
// $xml_string = XML document
// $xsl_file   = name of external xsl file
{
    //DebugBreak();
    // test that XSL funcions are available
    if (!class_exists('XsltProcessor')) {
        trigger_error('XSL functions are not available.', E_USER_ERROR);
    } // if
    
    global $mode;
    global $orderby;
    global $orderby_seq;
    global $title;
    global $task_id;
    
    set_error_handler('XML_errorHandler');
    
    // fetch the XSL stylesheet and load it into a DOM document
    $xsl = new DomDocument;
    $xsl->load("./xsl/$xsl_file");
    
    // create the XSLT processor and load the XSL file
    $xp = new XsltProcessor();
    $xp->importStylesheet($xsl);
    
    set_error_handler('errorHandler');
    
    $namespace = null;
    
    // set up optional parameters for XSL transformation
    $script = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'];
    $xp->setParameter($namespace, 'script', $script);
    
    if (isset($title)) {
        $xp->setParameter($namespace, 'title', $title);
    } // if
    if (isset($mode)) {
        $xp->setParameter($namespace, 'mode', $mode);
    } // if
    if (isset($task_id)) {
        $xp->setParameter($namespace, 'taskid', $task_id);
    } // if
    $xp->setParameter($namespace, 'session_name', session_name());
    
    if (isset($GLOBALS['subsys_dir'])) {
        $help_root = 'http://' . $_SERVER['HTTP_HOST'] . '/' . $GLOBALS['subsys_dir'];
    } else {
        $help_root = 'http://' . $_SERVER['HTTP_HOST'] . dirname($_SERVER['PHP_SELF']);
    } // if
    $xp->setParameter($namespace, 'help_root', $help_root);
    
    $doc_root = 'http://' . $_SERVER['HTTP_HOST'];
    $xp->setParameter($namespace, 'doc_root', $doc_root);
    
    // insert sort details
    if (ereg('\.', $orderby)) {
        // convert from 'table.column' to just 'column'
        list($table, $column) = split('\.', $orderby);
        $orderby = $column;
    } // if
    
    $xp->setParameter($namespace, 'orderby', $orderby);
    $xp->setParameter($namespace, 'order', $orderby_seq);
    
    // set charset to display accented characters correctly
    // header('content-type:text/html; charset=ISO-8859-1');
    header('content-type:text/html; charset=UTF-8'); 
    // disable any caching by the browser
    header('Expires: Mon, 14 Oct 2002 05:00:00 GMT'); // Date in the past
    header('Last-Modified: ' . gmdate("D, d M Y H:i:s") . ' GMT'); // always modified
    header('Cache-Control: no-store, no-cache, must-revalidate'); // HTTP 1.1
    header('Cache-Control: post-check=0, pre-check=0', false);
    header('Pragma: no-cache'); // HTTP 1.0
    
    // set the base directory name for 'xsl:include', 'xsl:import' etc
    //xslt_set_base($xp, 'file://' . dirname($_SERVER['SCRIPT_FILENAME']) . '/xsl/');
    
    // calculate the function's elapsed time
    list($usec, $sec) = explode(' ', microtime());
    $script_end = (float) $sec + (float) $usec;
    $xp->setParameter($namespace, 'script_time', round($script_end - $_SESSION['script_start'], 5));
    unset($_SESSION['script_start']);
    
    set_error_handler('XML_errorHandler');
    
    // transform the XML into HTML using the XSL file
    if ($html = $xp->transformToXML($xml_doc)) {
        echo $html;
    } else {
        trigger_error('XSL transformation failed.', E_USER_ERROR);
    } // if 
    
    set_error_handler('errorHandler');
    
    return;
    
} // XSLTransform

// ****************************************************************************
?>
