<?php 
// *****************************************************************************
// Copyright 2003-2004 by A J Marston <http://www.tonymarston.net>
// Distributed under the GNU General Public Licence
// *****************************************************************************

class Validation {

    // member variables
    var $caller;        // details of calling object
    var $errors;        // array of error messages
    
    // ****************************************************************************
    // class constructor
    // ****************************************************************************
    function Validation ()
    {
        
    } // Validation
    
    // ****************************************************************************
    function getErrors ()
    {
        return $this->errors;
        
    } // getErrors
    
    // ****************************************************************************
    function validateField ($fieldname, $fieldvalue, $fieldspec)
    // standard function for validating database fields
    {
        global $dateobj;

        if ($fieldspec['type'] == 'enum') {
            // get enum array for this field from the calling object
            $enum = $this->caller->getValRep($fieldname); 
            // if we already have the value do not replace it
            if (!in_array($fieldvalue, $enum)) {
                // replace index number with text value
                $fieldvalue = $enum[$fieldvalue];
            } // if
        } // if
        
        // trim any leading or trailing spaces
        $fieldvalue = trim($fieldvalue);
        
        if (strlen($fieldvalue) == 0) {
            // field is empty - is it allowed to be?
            if (isset($fieldspec['required'])) {
                $this->errors[$fieldname] = "$fieldname cannot be blank";
            } // if
            if ($fieldspec['type'] == 'date' AND isset($fieldspec['infinityisnull'])) {
                $fieldvalue = '9999-12-31';
            } // if 
            // nothing left to validate, so return now
            return $fieldvalue;
        } // if
        
        // field is not empty - check field size
        if (isset($fieldspec['size'])) {
            $size = (int)$fieldspec['size'];
            if (strlen($fieldvalue) > $size) {
                $this->errors[$fieldname] = "$fieldname cannot be > $size characters";
            } // if
        } // if
        
        switch ($fieldspec['type']) {
            case 'boolean': 
                // result from boolean fields may be varied, so convert to TRUE or FALSE
                // (where actual values are defined within $fieldspec)
                if (is_True($fieldvalue)) {
                    $fieldvalue = $fieldspec['true'];
                } else {
                    $fieldvalue = $fieldspec['false'];
                } // if
                break;
            
            case 'string':
                if (isset($fieldspec['uppercase'])) {
                    // value in this field must be uppercase
                    $fieldvalue = strtoupper($fieldvalue);
                } // if
                if (isset($fieldspec['lowercase'])) {
                    // value in this field must be lowercase
                    $fieldvalue = strtolower($fieldvalue);
                } // if
                
                // escape any suspect characters in string fields
                $fieldvalue = addslashes($fieldvalue);
                
                break;
            
            case 'date': 
                break;
            
            case 'datetime': 
                break;
            
            case 'time': 
                break;
            
            default: 
                // perform validation if field type is numeric (integer, decimal)
                //$fieldvalue = $this->validateNumber($fieldname, $fieldvalue, $fieldspec);
        } // switch
        
        return $fieldvalue;
        
    } // validateField
    
    // ****************************************************************************
    function validateUpdateArray ($fieldarray, $fieldspec, &$caller)
    // validate contents of $fieldarray against $fieldspec array.
    // errors are returned in $errors array.
    // NOTE: for UPDATE only a subset of fields may be supplied.
    {
        $this->errors = array();
        
        $this->caller = &$caller;
        
        // create array to hold data which has been formatted for the database
        $updatearray = array();
        
        // step through input data and compare with fieldspec
        foreach ($fieldarray as $field => $value) {
            // get specifications for this field
            // (this will not carry forward any fields which do not exist in this table)
            if (array_key_exists($field, $fieldspec)) {
                $spec = $fieldspec[$field];
                
                $value = $this->validateField($field, $value, $spec); 
                // transfer to array which will be passed to the database
                // (allow null values as field may have been cleared)
                if (strlen($value) > 0) {
                    $updatearray[$field] = $value;
                } else {
                    $updatearray[$field] = null;
                } // if
            } // if
        } // foreach
        
        return $updatearray;
        
    } // validateUpdateArray
    
// ****************************************************************************
} // end class
// ****************************************************************************

?>
