<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<!--
//*****************************************************************************
// Copyright 2003-2004 by A J Marston <http://www.tonymarston.net>
// Distributed under the GNU General Public Licence
//*****************************************************************************
-->

<xsl:param name="orderby"></xsl:param>
<xsl:param name="order"></xsl:param>
<xsl:param name="numrows">2</xsl:param>

<!-- create <colgroup> entries -->
<xsl:template name="column_group">
  <xsl:param name="table"/>
  
  <xsl:for-each select="//structure/*[name()=$table]/columns/column">
    <colgroup>
      <xsl:if test="@width">
        <xsl:attribute name="width" ><xsl:value-of select="@width" /></xsl:attribute>
      </xsl:if>
      <xsl:if test="@class">
        <xsl:attribute name="class" ><xsl:value-of select="@class" /></xsl:attribute>
      </xsl:if>
    </colgroup>
  </xsl:for-each>

</xsl:template>


<!-- set up the column headings using fieldlist in XML file -->
<xsl:template name="column_headings">
  <xsl:param name="table"/>
  
  <tr>
    <xsl:for-each select="//structure/*[name()=$table]/fieldlist/field">
      <th>
        <xsl:call-template name="column_hdg">
          <xsl:with-param name="item" select="@id"/>
          <xsl:with-param name="label" select="@label"/>
          <xsl:with-param name="nosort" select="@nosort"/>
        </xsl:call-template>
      </th>
    </xsl:for-each>
  </tr>
  
</xsl:template>


<!-- create a single column heading -->
<xsl:template name="column_hdg">
  <xsl:param name="item"/>
  <xsl:param name="label"/>
  <xsl:param name="nosort"/>
  
  <xsl:variable name="session" select="concat('session_name=',$session_name)" />

  <xsl:choose>
  
    <xsl:when test="$item='selectbox'">
      <!-- text only, no hyperlink -->
      <xsl:value-of select="$label"/>
    </xsl:when>
    
    <xsl:when test="$numrows > 0 and not($nosort)">
      <!-- $numrows is one of the XSL parameters -->
      <!-- note that if 'nosort' is set there are no hyperlinks for sorting -->
    
      <!-- create hyperlink to sort on this field -->
      <a href="{$script}?{$session}&amp;orderby={$item}">
        <!-- this is the visible text for the hyperlink -->
        <xsl:value-of select="$label"/>
      </a>
      <!-- if sorted by this field insert ascending or descending image -->
      <xsl:if test="$orderby=$item">
        <img src="images/order_{$order}.gif" height="16" width="16" alt="order_{$order}.gif" />
      </xsl:if>
      
    </xsl:when>
    
    <xsl:otherwise>
      <!-- no sorting allowed, so don't bother with the hyperlink -->
      <xsl:value-of select="$label"/>
    </xsl:otherwise>
    
  </xsl:choose>
  
</xsl:template>

</xsl:stylesheet>
