<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<!--
//*****************************************************************************
// Copyright 2003-2004 by A J Marston <http://www.tonymarston.net>
// Distributed under the GNU General Public Licence
//*****************************************************************************
-->

<xsl:include href="std.replace.xsl"/>

<xsl:template name="menubar">
  
  <xsl:variable name="session" select="concat('session_name=',$session_name)" />
  
  <div class="menubar">
    <table class="menubar" cellspacing="0" cellpadding="0">
    
      <!-- this row has one cell foreach entry, plus a blank cell at the end -->
      <tr class="menubar">
      
        <xsl:for-each select="//menubar/button">
          <td>
          
            <xsl:choose>
              <xsl:when test="@active">
                <xsl:attribute name="class">active</xsl:attribute>
              </xsl:when>
              <xsl:otherwise>
                <xsl:attribute name="class">inactive</xsl:attribute>
              </xsl:otherwise>
            </xsl:choose>
            
            <!-- create a link for each element within navbar -->
            <a href="{$help_root}/{@id}?{$session}">
              <!-- replace ordinary space with non-breaking space -->
              <xsl:call-template name="replace">
                <xsl:with-param name="text" select="node()"/>
                <xsl:with-param name="replace" select="' '"/>
                <xsl:with-param name="by" select="'&#160;'"/>
              </xsl:call-template>
            </a>
            
          </td>
        </xsl:for-each>
        
        <td class="filler">&#160;</td> <!-- blank cell to take up all the remaining space -->
        
      </tr>
      
      <!-- this row is the same colour as the active cell in the previous row -->
      <!-- it also contains entries for each page in the current hierarchy (aka 'breadcrumbs') -->
      <tr class="menustack">
      
        <td class="active">
          <xsl:attribute name="colspan">
            <!-- include the blank cell at the end -->
            <xsl:value-of select="count(//menubar/button)+1"/>
          </xsl:attribute>
          
          <xsl:for-each select="//menubar/stack">
          
            <xsl:choose>
              <xsl:when test="position()=last() or @active">
                <!-- last/active entry is not a hyperlink, just plain text -->
                <xsl:value-of select="node()"/>
              </xsl:when>
              <xsl:otherwise>
                <!-- insert hyperlink -->
                <a href="{$script}?selection={@id}&amp;{$session}">
                  <xsl:value-of select="node()"/>
                </a>
              </xsl:otherwise>
            </xsl:choose>
            
            <xsl:if test="not(position()=last())">
              <!-- not last entry, so insert a '>>' separator before the next entry -->
              <xsl:text>&#187;</xsl:text>
            </xsl:if>
            
          </xsl:for-each>
          
        </td>
      </tr>
      
    </table>
  </div>

</xsl:template>

</xsl:stylesheet>
