<?xml version='1.0'?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<!--
//*****************************************************************************
// Copyright 2003-2004 by A J Marston <http://www.tonymarston.net>
//*****************************************************************************
-->

<!--
//*****************************************************************************
// PAGINATION - create links to jump to first/prev/next/last page of current set
//*****************************************************************************
-->
<xsl:template name="pagination">
  <xsl:param name="object" />
  
  <xsl:variable name="session" select="concat('session_name=',$session_name)" />
  
  <div class="pagination">
    <table class="pagination">
      <tr class="pagination">
      
        <!-- look for the entry with the specified value in the id attribute -->
        <xsl:for-each select="//pagination/page[@id=$object]">
        
          <xsl:choose>
            <xsl:when test="@curpage&lt;=1">
              <!-- we are on page 1, so there is no navigation backwards -->
              <td class="pagination">&#171;FIRST</td>
              <td class="pagination">&#8249;PREV</td>
            </xsl:when>
            <xsl:otherwise>
              <!-- insert links for first/previous page -->
              <td class="pagination"><a href="{$script}?{$session}&amp;pagination={$object}&amp;page=1"><b>&#171;FIRST</b></a></td>
              <td class="pagination"><a href="{$script}?{$session}&amp;pagination={$object}&amp;page={@curpage -1}"><b>&#8249;PREV</b></a></td>
            </xsl:otherwise>
          </xsl:choose>
          
          <!-- insert "page x of y" -->
          <td class="pagination">
            <xsl:text>(Page </xsl:text> 
            <xsl:value-of select="@curpage"/>
            <xsl:text> of </xsl:text> 
            <xsl:value-of select="@lastpage"/>
            <xsl:text>)</xsl:text>
          </td>
          
          <xsl:choose>
            <xsl:when test="@curpage=@lastpage">
              <!-- we are on the last page, so there is no navigation forwards -->
              <td class="pagination">NEXT&#8250;</td>
              <td class="pagination">LAST&#187;</td>
            </xsl:when>
            <xsl:otherwise>
              <!-- insert links for last/next item -->
              <td class="pagination"><a href="{$script}?{$session}&amp;pagination={$object}&amp;page={@curpage +1}"><b>NEXT&#8250;</b></a></td>
              <td class="pagination"><a href="{$script}?{$session}&amp;pagination={$object}&amp;page={@lastpage}"><b>LAST&#187;</b></a></td>
            </xsl:otherwise>
          </xsl:choose>
          
        </xsl:for-each>
        
      </tr>
    </table>
  </div>
  
</xsl:template> <!-- PAGINATION -->

<!--
//*****************************************************************************
// SCROLLING - create links to jump to first/prev/next/last item of current set
//*****************************************************************************
-->
<xsl:template name="scrolling">
  <xsl:param name="object" />
  
  <xsl:variable name="session" select="concat('session_name=',$session_name)" />
  
  <div class="scrolling">
  
    <xsl:text> </xsl:text>  <!-- insert a space to prevent an empty element -->
    
    <!-- look for the entry with the specified value in the id attribute -->
    <xsl:for-each select="//scrolling/scroll[@id=$object]">
    
      <!-- include only if there is more than 1 item -->
      <xsl:if test="@lastitem&gt;1">
      
        <table class="scrolling">
          <tr class="scrolling">
          
            <xsl:choose>
              <xsl:when test="@curitem&lt;=1">
                <!-- we are on item 1, so there is no navigation backwards -->
                <td class="scrolling">&#171;FIRST</td>
                <td class="scrolling">&#8249;PREV</td>
              </xsl:when>
              <xsl:otherwise>
                <!-- insert links for first/previous item -->
                <td class="scrolling"><a href="{$script}?{$session}&amp;scrolling={$object}&amp;item=1"><b>&#171;FIRST</b></a></td>
                <td class="scrolling"><a href="{$script}?{$session}&amp;scrolling={$object}&amp;item={@curitem -1}"><b>&#8249;PREV</b></a></td>
              </xsl:otherwise>
            </xsl:choose>
          
            <!-- insert "item x of y" -->
            <td class="scrolling">
              <xsl:text>Item </xsl:text> 
              <xsl:value-of select="@curitem"/>
              <xsl:text> of </xsl:text> 
              <xsl:value-of select="@lastitem"/>
            </td>
          
            <xsl:choose>
              <xsl:when test="@curitem=@lastitem">
                <!-- we are on the last item, so there is no navigation forwards -->
                <td class="scrolling">NEXT&#8250;</td>
                <td class="scrolling">LAST&#187;</td>
              </xsl:when>
              <xsl:otherwise>
                <!-- insert links for last/next item -->
                <td class="scrolling"><a href="{$script}?{$session}&amp;scrolling={$object}&amp;item={@curitem +1}"><b>NEXT&#8250;</b></a></td>
                <td class="scrolling"><a href="{$script}?{$session}&amp;scrolling={$object}&amp;item={@lastitem}"><b>LAST&#187;</b></a></td>
              </xsl:otherwise>
            </xsl:choose>
            
          </tr>
        </table>
        
      </xsl:if>
      
    </xsl:for-each>
    
  </div>
  
</xsl:template> <!-- SCROLLING -->

</xsl:stylesheet>
